/*
    Project: TCTEMP
    Version: V1.17
    File   : na_filedevio.c

    Copyright (c) 2006-2009 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "na_priv.h"

BOOL NAAPI NaDeviceIoControl(HANDLE hFile, ULONG ulControlCode,
  PVOID pInputBuffer, ULONG ulInputBufferLength, PVOID pOutputBuffer,
  ULONG ulOutputBufferLength, PULONG pulBytesReturned)
{
    NTSTATUS        ntStatus;
    IO_STATUS_BLOCK isb;

    if (HIWORD(ulControlCode) != FILE_DEVICE_FILE_SYSTEM) {
        ntStatus = NtDeviceIoControlFile(hFile, NULL, NULL, NULL,
          &isb, ulControlCode, pInputBuffer, ulInputBufferLength, pOutputBuffer,
          ulOutputBufferLength);
    } else {
        ntStatus = NtFsControlFile(hFile, NULL, NULL, NULL,
          &isb, ulControlCode, pInputBuffer, ulInputBufferLength, pOutputBuffer,
          ulOutputBufferLength);
    }
    if (ntStatus == STATUS_PENDING) {
        if (!NaWaitForFile(hFile, NULL)) {
            NaDbgPrintf(L"WaitForFile failed in DeviceIoControl!");
            return FALSE;
        }
        ntStatus = isb.Status;
        if (!NT_SUCCESS(ntStatus)) {
            NaDbgPrintf(
              L"DeviceIoControl failed after waiting (0x%08x)",
                ntStatus);
            return FALSE;
        }
    } else if (!NT_SUCCESS(ntStatus)) {
#if DEBUG
        if (ulControlCode != FSCTL_ALLOW_EXTENDED_DASD_IO) {
            NaDbgPrintf(L"DeviceIoControl failed (0x%08x)", ntStatus);
        }
#endif
        return FALSE;
    }

    if (pulBytesReturned) {
        *pulBytesReturned = (ULONG) isb.Information;
    }

    return TRUE;
}
